﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using C1.Chart;
using AxisGroupingDemo.Data.Model;

namespace AxisGroupingDemo.View
{
    /// <summary>
    /// Interaction logic for DateTimeAxisGrouping.xaml
    /// </summary>
    public partial class DateTimeAxisGrouping : UserControl
    {
        private Random rnd;
        public DateTimeAxisGrouping()
        {
            InitializeComponent();
            rnd = new Random();
            flexChart.ItemsSource = CreateData();
        }
        List<Quote> CreateData()
        {
            List<Quote> list = new List<Quote>();
            var dt = DateTime.Today;
            var price = 1000;
            for (var i = 0; i < 365; i++)
            {
                price += rnd.Next(10) % 2 == 0 ? rnd.Next(50) : -rnd.Next(50);
                list.Add(new Quote() { Time = dt.AddDays(i), Price = price });
            }

            return list;
        }

        private void chbAllowGrouping_Checked(object sender, RoutedEventArgs e)
        {
            if (flexChart != null)
            {
                if (chbAllowGrouping.IsChecked == true)
                {
                    (flexChart.AxisX.GroupProvider as DateTimeGroupProvider).AllowGrouping = true;
                    flexChart.AxisX.Format = "dd";
                    flexChart.AxisX.LabelAngle = double.NaN;
                }
                else
                {
                    (flexChart.AxisX.GroupProvider as DateTimeGroupProvider).AllowGrouping = false;
                    flexChart.AxisX.Format = "dd-MMM-yyyy";
                    flexChart.AxisX.LabelAngle = 90;
                }
                flexChart.Invalidate();
            }
        }

        private void OnDateTimeGroupChanged(object sender, RoutedEventArgs e)
        {
            if (flexChart != null)
            {
                CheckBox chb = sender as CheckBox;
                var showGroup = (DateTimeGroup)Enum.Parse(typeof(DateTimeGroup),chb.Tag.ToString());
                var groupProvider = flexChart.AxisX.GroupProvider as DateTimeGroupProvider;
                if (chb.IsChecked == true)
                    groupProvider.ShownGroups = groupProvider.ShownGroups | showGroup;
                else
                    groupProvider.ShownGroups = groupProvider.ShownGroups ^ showGroup;
                flexChart.Invalidate();
            }
        }
    }
    [Flags]
    public enum DateTimeGroup
    {
        Month = 1,
        Quarter = 2,
        Year = 4
    }

    public class DateTimeGroupProvider : IAxisGroupProvider
    {
        public DateTimeGroup ShownGroups { get; set; }

        public bool AllowGrouping { get; set; }

        public DateTimeGroupProvider()
        {
            AllowGrouping = true;
            ShownGroups = DateTimeGroup.Month | DateTimeGroup.Quarter | DateTimeGroup.Year;
        }

        public int GetLevels(IRange range)
        {
            return AllowGrouping ? 3 : 0;
        }

        public IList<IRange> GetRanges(IRange range, int level)
        {
            var timeRange = range as TimeRange;
            if (timeRange == null)
                return null;
            var min = timeRange.TimeMin;
            var max = timeRange.TimeMax;

            var span = max - min;

            List<IRange> ranges = new List<IRange>();
            DateTime start;
            switch (level)
            {
                case 1:
                    if (!ShownGroups.HasFlag(DateTimeGroup.Month))
                        return null;
                    start = new DateTime(min.Year, min.Month, 1);
                    ranges = Enumerable.Range(0, ((max.Year - min.Year) * 12) + max.Month - min.Month + 1).Select(a => start.AddMonths(a))
                   .TakeWhile(a => a <= max)
                   .Select(a => (IRange)(new TimeRange(a.ToString("MMM"), a, a.AddMonths(1)))).ToList();

                    break;
                case 2:
                    if (!ShownGroups.HasFlag(DateTimeGroup.Quarter))
                        return null;
                    start = new DateTime(min.Year, ((int)Math.Ceiling((double)min.Month / 3) - 1) * 3 + 1, 1);
                    ranges = Enumerable.Range(0, ((max.Month - start.Month) / 3 + 1) + 4 * (max.Year - start.Year)).Select(a => start.AddMonths(a * 3))
                   .TakeWhile(a => a <= max)
                   .Select(a => (IRange)(new TimeRange("Q" + (int)Math.Ceiling((double)a.Month / 3), a, a.AddMonths(3)))).ToList();
                    break;
                case 3:
                    if (!ShownGroups.HasFlag(DateTimeGroup.Year))
                        return null;
                    start = new DateTime(min.Year, 1, 1);
                    ranges = Enumerable.Range(0, max.Year - start.Year + 1).Select(a => start.AddYears(a))
                   .TakeWhile(a => a <= max)
                   .Select(a => (IRange)(new TimeRange(a.ToString("yyyy"), a, a.AddYears(1)))).ToList();
                    break;
            }
            return ranges;
        }
    }
}
