﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using C1.Chart;
using AxisGroupingDemo.Data;

namespace AxisGroupingDemo.View
{
    /// <summary>
    /// Interaction logic for NumericalAxisGrouping.xaml
    /// </summary>
    public partial class NumericalAxisGrouping : UserControl
    {
        public NumericalAxisGrouping()
        {
            InitializeComponent();
            cbStates.ItemsSource = DataService.GetAQIData();
        }

        private void C1NumericBox_ValueChanged(object sender, C1.WPF.PropertyChangedEventArgs<double> e)
        {
            if (flexChart != null)
            {
                var groupProvider = flexChart.AxisY.GroupProvider as NumericGroupProvider;
                groupProvider.Levels = (int)nbLevels.Value;
                flexChart.Invalidate();
            }
        }
    }
    public class NumericGroupProvider : IAxisGroupProvider
    {
        public enum Pollutant
        {
            PM10,
            PM25
        }

        public int Levels { get; set; }

        public NumericGroupProvider()
        {
            Levels = 1;
        }

        public Pollutant PollutantType { get; set; }

        public int GetLevels(IRange range)
        {
            return Levels;
        }

        public IList<IRange> GetRanges(IRange range, int level)
        {
            var ranges = new List<IRange>();
            if (level == 1)
            {
                if (PollutantType == Pollutant.PM10)
                {
                    ranges.Add(new DoubleRange("Good", 0, 54));
                    ranges.Add(new DoubleRange("Moderate", 55, 154));
                    ranges.Add(new DoubleRange("Unhealthy", 155, 354));
                    ranges.Add(new DoubleRange("Very Unhealthy", 355, 424));
                    ranges.Add(new DoubleRange("Hazardous", 425, double.PositiveInfinity));
                }
                else
                {
                    ranges.Add(new DoubleRange("Good", 0, 12));
                    ranges.Add(new DoubleRange("Moderate", 12.1, 35.4));
                    ranges.Add(new DoubleRange("Unhealthy", 35.5, 150.4));
                    ranges.Add(new DoubleRange("Very Unhealthy", 150.5, 250.4));
                    ranges.Add(new DoubleRange("Hazardous", 250.5, double.PositiveInfinity));
                }
            }
            return ranges;
        }
    }
}
